/*
   This file is part of tiling.

   tiling is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   tiling is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with tiling.  If not, see <http://www.gnu.org/licenses/>.

   Copyright (C) 2010 Neil Birkbeck
*/
/* 
   Simple interface for delaunay triangulation.
   
   Copyright (c) 2009, Neil Birkbeck      
*/

#ifndef NACB_DELAUNAY_H
#include <nmath/vec2.h>
#include <nmath/vec3.h>
#include <vector>

/** \class Circle structure, holding point indices and whether event is valid.
 */
struct circle_t{
  int p1, p2, p3;
  bool false_alarm;

  circle_t(int p1, int p2, int p3){
    this->p1 = p1;
    this->p2 = p2;
    this->p3 = p3;
    false_alarm = false;
  }
};

/** A segment on the beach line, described by its point, and containing
    a possible reference to a circle.
*/
struct segment_t{
  int point;
  circle_t * circle;
  segment_t(int pt = -1, circle_t * cv = 0){
    point = pt;
    circle = cv;
  }
};

/** Passed around for interactive display.  Not needed otherwise.
 */
struct delaunay_state_t{
  std::vector<segment_t> *  segs;
  std::vector<nacb::Vec3<int> > * tris;
  double              sweep;
  double               next;
  int                 point;
  delaunay_state_t(std::vector<segment_t> * s,
		   std::vector<nacb::Vec3<int> > * t,
		   double sw, double n, int pt = -1){
    segs = s;
    tris = t;
    sweep = sw;
    next  = n;
    point = pt;
  }
};

//Callback for interactive displays (or quitting midway).
typedef bool (* interactions_func_t)(int location, const delaunay_state_t &);

/** Solve the quadratic equation and return appropriate solution given
    order of p1 and p2.
    \param p1 The site location of the first site arc
    \param p2 The site location of the second sites arc
    \return The x coordinate of solution.
*/
double solve(double yl, const nacb::Vec2d & p1, const nacb::Vec2d & p2);

nacb::Vec2d center(const nacb::Vec2d & p1,
		   const nacb::Vec2d & p2,
		   const nacb::Vec2d & p3);

double eval(double liney, const nacb::Vec2d & pi, double x);

std::vector<nacb::Vec3<int> > delaunay(std::vector<nacb::Vec2d> & pts,
				       interactions_func_t interactions = 0);

#endif //end NACB_DELAUNAY_H
